@@ -6,9 +6,9 @@
     Assumes buffer size in bytes is a multiple of 16
     Assumes SDL_MIX_MAXVOLUME = 128
 */
+#include "SDL_config.h"
 
-
-#if defined(i386) && defined(__GNUC__) && defined(USE_ASMBLIT)
+#if defined(__GNUC__) && defined(__i386__) && defined(SDL_ASSEMBLY_ROUTINES)
 void SDL_MixAudio_MMX_S16(char* ,char* ,unsigned int ,int );
 void SDL_MixAudio_MMX_S8(char* ,char* ,unsigned int ,int );
 #endif
