@@ -19,6 +19,7 @@
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
 /*
 	Audio interrupt variables and callback function
@@ -26,16 +27,13 @@
 	Patrice Mandin
 */
 
-#include <string.h>
 #include <unistd.h>
 
 #include <mint/osbind.h>
 #include <mint/falcon.h>
 #include <mint/mintbind.h>
 
-#include "SDL_types.h"
 #include "SDL_audio.h"
-
 #include "SDL_mintaudio.h"
 #include "SDL_mintaudio_stfa.h"
 
@@ -44,9 +42,9 @@
 SDL_AudioDevice *SDL_MintAudio_device;
 Uint8 *SDL_MintAudio_audiobuf[2];	/* Pointers to buffers */
 unsigned long SDL_MintAudio_audiosize;		/* Length of audio buffer=spec->size */
-unsigned short SDL_MintAudio_numbuf;		/* Buffer to play */
-unsigned short SDL_MintAudio_mutex;
-unsigned long SDL_MintAudio_clocktics;
+volatile unsigned short SDL_MintAudio_numbuf;		/* Buffer to play */
+volatile unsigned short SDL_MintAudio_mutex;
+volatile unsigned long SDL_MintAudio_clocktics;
 cookie_stfa_t	*SDL_MintAudio_stfa;
 
 /* MiNT thread variables */
@@ -63,7 +61,7 @@ void SDL_MintAudio_Callback(void)
 	SDL_AudioDevice *audio = SDL_MintAudio_device;
 
  	buffer = SDL_MintAudio_audiobuf[SDL_MintAudio_numbuf];
-	memset(buffer, audio->spec.silence, audio->spec.size);
+	SDL_memset(buffer, audio->spec.silence, audio->spec.size);
 
 	if (audio->paused)
 		return;
@@ -76,11 +74,11 @@ void SDL_MintAudio_Callback(void)
 		} else {
 			silence = 0;
 		}
-		memset(audio->convert.buf, silence, audio->convert.len);
+		SDL_memset(audio->convert.buf, silence, audio->convert.len);
 		audio->spec.callback(audio->spec.userdata,
 			(Uint8 *)audio->convert.buf,audio->convert.len);
 		SDL_ConvertAudio(&audio->convert);
-		memcpy(buffer, audio->convert.buf, audio->convert.len_cvt);
+		SDL_memcpy(buffer, audio->convert.buf, audio->convert.len_cvt);
 	} else {
 		audio->spec.callback(audio->spec.userdata, buffer, audio->spec.size);
 	}
@@ -106,7 +104,7 @@ void SDL_MintAudio_AddFrequency(_THIS, Uint32 frequency, Uint32 clock,
 	/* Put all following ones farer */
 	if (MINTAUDIO_freqcount>0) {
 		for (i=MINTAUDIO_freqcount; i>p; i--) {
-			memcpy(&MINTAUDIO_frequencies[i], &MINTAUDIO_frequencies[i-1], sizeof(mint_frequency_t));
+			SDL_memcpy(&MINTAUDIO_frequencies[i], &MINTAUDIO_frequencies[i-1], sizeof(mint_frequency_t));
 		}
 	}
 
