@@ -1,34 +1,29 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_quit.c,v 1.6 2005/08/21 06:18:54 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* General quit handling code for SDL */
 
-#include <stdio.h>
-#ifndef NO_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
 
@@ -36,7 +31,7 @@ static char rcsid =
 #include "SDL_events_c.h"
 
 
-#ifndef NO_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 static void SDL_HandleSIG(int sig)
 {
 	/* Reset the signal handler */
@@ -45,12 +40,12 @@ static void SDL_HandleSIG(int sig)
 	/* Signal a quit interrupt */
 	SDL_PrivateQuit();
 }
-#endif /* NO_SIGNAL_H */
+#endif /* HAVE_SIGNAL_H */
 
 /* Public functions */
 int SDL_QuitInit(void)
 {
-#ifndef NO_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 	void (*ohandler)(int);
 
 	/* Both SIGINT and SIGTERM are translated into quit interrupts */
@@ -60,14 +55,14 @@ int SDL_QuitInit(void)
 	ohandler = signal(SIGTERM, SDL_HandleSIG);
 	if ( ohandler != SIG_DFL )
 		signal(SIGTERM, ohandler);
-#endif /* NO_SIGNAL_H */
+#endif /* HAVE_SIGNAL_H */
 
 	/* That's it! */
 	return(0);
 }
 void SDL_QuitQuit(void)
 {
-#ifndef NO_SIGNAL_H
+#ifdef HAVE_SIGNAL_H
 	void (*ohandler)(int);
 
 	ohandler = signal(SIGINT, SIG_DFL);
@@ -76,7 +71,7 @@ void SDL_QuitQuit(void)
 	ohandler = signal(SIGTERM, SIG_DFL);
 	if ( ohandler != SDL_HandleSIG )
 		signal(SIGTERM, ohandler);
-#endif /* NO_SIGNAL_H */
+#endif /* HAVE_SIGNAL_H */
 }
 
 /* This function returns 1 if it's okay to close the application window */
