@@ -19,15 +19,15 @@
 	Sam Lantinga
 	slouken@libsdl.org
 */
+#include "SDL_config.h"
 
-/* SDL joystick driver for Darwin / MacOS X, based on the IOKit HID API */
+#ifdef SDL_JOYSTICK_IOKIT
+
+/* SDL joystick driver for Darwin / Mac OS X, based on the IOKit HID API */
 /* Written 2001 by Max Horn */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <sys/errno.h>
 #include <sysexits.h>
 #include <mach/mach.h>
 #include <mach/mach_error.h>
@@ -36,7 +36,7 @@
 #ifdef MACOS_10_0_4
 #include <IOKit/hidsystem/IOHIDUsageTables.h>
 #else
-/* The header was moved here in MacOS X 10.1 */
+/* The header was moved here in Mac OS X 10.1 */
 #include <Kernel/IOKit/hidsystem/IOHIDUsageTables.h>
 #endif
 #include <IOKit/hid/IOHIDLib.h>
@@ -44,49 +44,48 @@
 #include <CoreFoundation/CoreFoundation.h>
 #include <Carbon/Carbon.h> /* for NewPtrClear, DisposePtr */
 
-#include "SDL_error.h"
 #include "SDL_joystick.h"
-#include "SDL_sysjoystick.h"
-#include "SDL_joystick_c.h"
+#include "../SDL_sysjoystick.h"
+#include "../SDL_joystick_c.h"
 
 struct recElement
 {
-	IOHIDElementCookie cookie;				// unique value which identifies element, will NOT change
-	long min;								// reported min value possible
-	long max;								// reported max value possible
-/*
-	TODO: maybe should handle the following stuff somehow?
+	IOHIDElementCookie cookie;				/* unique value which identifies element, will NOT change */
+	long min;								/* reported min value possible */
+	long max;								/* reported max value possible */
+#if 0
+	/* TODO: maybe should handle the following stuff somehow? */
 
-	long scaledMin;							// reported scaled min value possible
-	long scaledMax;							// reported scaled max value possible
-	long size;								// size in bits of data return from element
-	Boolean relative;						// are reports relative to last report (deltas)
-	Boolean wrapping;						// does element wrap around (one value higher than max is min)
-	Boolean nonLinear;						// are the values reported non-linear relative to element movement
-	Boolean preferredState;					// does element have a preferred state (such as a button)
-	Boolean nullState;						// does element have null state
-*/
+	long scaledMin;							/* reported scaled min value possible */
+	long scaledMax;							/* reported scaled max value possible */
+	long size;								/* size in bits of data return from element */
+	Boolean relative;						/* are reports relative to last report (deltas) */
+	Boolean wrapping;						/* does element wrap around (one value higher than max is min) */
+	Boolean nonLinear;						/* are the values reported non-linear relative to element movement */
+	Boolean preferredState;					/* does element have a preferred state (such as a button) */
+	Boolean nullState;						/* does element have null state */
+#endif /* 0 */
 
 	/* runtime variables used for auto-calibration */
-	long minReport;							// min returned value
-	long maxReport;							// max returned value
+	long minReport;							/* min returned value */
+	long maxReport;							/* max returned value */
 	
-	struct recElement * pNext;				// next element in list
+	struct recElement * pNext;				/* next element in list */
 };
 typedef struct recElement recElement;
 
 struct joystick_hwdata
 {
-	IOHIDDeviceInterface ** interface;		// interface to device, NULL = no interface
+	IOHIDDeviceInterface ** interface;		/* interface to device, NULL = no interface */
 
-	char product[256];							// name of product
-	long usage;								// usage page from IOUSBHID Parser.h which defines general usage
-	long usagePage;							// usage within above page from IOUSBHID Parser.h which defines specific usage
+	char product[256];							/* name of product */
+	long usage;								/* usage page from IOUSBHID Parser.h which defines general usage */
+	long usagePage;							/* usage within above page from IOUSBHID Parser.h which defines specific usage */
 
-	long axes;								// number of axis (calculated, not reported by device)
-	long buttons;							// number of buttons (calculated, not reported by device)
-	long hats;								// number of hat switches (calculated, not reported by device)
-	long elements;							// number of total elements (shouldbe total of above) (calculated, not reported by device)
+	long axes;								/* number of axis (calculated, not reported by device) */
+	long buttons;							/* number of buttons (calculated, not reported by device) */
+	long hats;								/* number of hat switches (calculated, not reported by device) */
+	long elements;							/* number of total elements (shouldbe total of above) (calculated, not reported by device) */
 
 	recElement* firstAxis;
 	recElement* firstButton;
@@ -95,7 +94,7 @@ struct joystick_hwdata
 	int removed;
 	int uncentered;
 
-	struct joystick_hwdata* pNext;			// next device
+	struct joystick_hwdata* pNext;			/* next device */
 };
 typedef struct joystick_hwdata recDevice;
 
@@ -134,32 +133,17 @@ static SInt32 HIDGetElementValue (recDevice *pDevice, recElement *pElement)
 		}
 	}
 
-	// auto user scale
+	/* auto user scale */
 	return hidEvent.value;
 }
 
-/* similiar to HIDGetElementValue, but auto-calibrates the value before returning it */
-
-static SInt32 HIDCalibratedValue (recDevice *pDevice, recElement *pElement)
-{
-	float deviceScale = pElement->max - pElement->min;
-	float readScale = pElement->maxReport - pElement->minReport;
-	SInt32 value = HIDGetElementValue(pDevice, pElement);
-	if (readScale == 0)
-		return value; // no scaling at all
-	else
-		return ((value - pElement->minReport) * deviceScale / readScale) + pElement->min;
-}
-
-/* similiar to HIDCalibratedValue but calibrates to an arbitrary scale instead of the elements default scale */
-
 static SInt32 HIDScaledCalibratedValue (recDevice *pDevice, recElement *pElement, long min, long max)
 {
 	float deviceScale = max - min;
 	float readScale = pElement->maxReport - pElement->minReport;
 	SInt32 value = HIDGetElementValue(pDevice, pElement);
 	if (readScale == 0)
-		return value; // no scaling at all
+		return value; /* no scaling at all */
 	else
 		return ((value - pElement->minReport) * deviceScale / readScale) + min;
 }
@@ -194,7 +178,7 @@ static IOReturn HIDCreateOpenDeviceInterface (io_object_t hidDevice, recDevice *
 													kIOCFPlugInInterfaceID, &ppPlugInInterface, &score);
 		if (kIOReturnSuccess == result)
 		{
-			// Call a method of the intermediate plug-in to create the device interface
+			/* Call a method of the intermediate plug-in to create the device interface */
 			plugInResult = (*ppPlugInInterface)->QueryInterface (ppPlugInInterface,
 								CFUUIDGetUUIDBytes (kIOHIDDeviceInterfaceID), (void *) &(pDevice->interface));
 			if (S_OK != plugInResult)
@@ -228,15 +212,15 @@ static IOReturn HIDCloseReleaseInterface (recDevice *pDevice)
 	
 	if ((NULL != pDevice) && (NULL != pDevice->interface))
 	{
-		// close the interface
+		/* close the interface */
 		result = (*(pDevice->interface))->close (pDevice->interface);
 		if (kIOReturnNotOpen == result)
 		{
-			//  do nothing as device was not opened, thus can't be closed
+			/* do nothing as device was not opened, thus can't be closed */
 		}
 		else if (kIOReturnSuccess != result)
 			HIDReportErrorNum ("Failed to close IOHIDDeviceInterface.", result);
-		//release the interface
+		/* release the interface */
 		result = (*(pDevice->interface))->Release (pDevice->interface);
 		if (kIOReturnSuccess != result)
 			HIDReportErrorNum ("Failed to release IOHIDDeviceInterface.", result);
@@ -562,7 +546,7 @@ static recDevice *HIDDisposeDevice (recDevice **ppDevice)
 	recDevice *pDeviceNext = NULL;
 	if (*ppDevice)
 	{
-		// save next device prior to disposing of this device
+		/* save next device prior to disposing of this device */
 		pDeviceNext = (*ppDevice)->pNext;
 		
 		/* free element lists */
@@ -588,15 +572,15 @@ static recDevice *HIDDisposeDevice (recDevice **ppDevice)
 int SDL_SYS_JoystickInit(void)
 {
 	IOReturn result = kIOReturnSuccess;
-	mach_port_t masterPort = NULL;
-	io_iterator_t hidObjectIterator = NULL;
+	mach_port_t masterPort = 0;
+	io_iterator_t hidObjectIterator = 0;
 	CFMutableDictionaryRef hidMatchDictionary = NULL;
 	recDevice *device, *lastDevice;
-	io_object_t ioHIDDeviceObject = NULL;
+	io_object_t ioHIDDeviceObject = 0;
 	
 	SDL_numjoysticks = 0;
 	
-	if (NULL != gpDeviceList)
+	if (gpDeviceList)
 	{
 		SDL_SetError("Joystick: Device list already inited.");
 		return -1;
@@ -611,7 +595,7 @@ int SDL_SYS_JoystickInit(void)
 
 	/* Set up a matching dictionary to search I/O Registry by class name for all HID class devices. */
 	hidMatchDictionary = IOServiceMatching (kIOHIDDeviceKey);
-	if ((hidMatchDictionary != NULL))
+	if (hidMatchDictionary)
 	{
 		/* Add key for device type (joystick, in this case) to refine the matching dictionary. */
 		
@@ -640,7 +624,7 @@ int SDL_SYS_JoystickInit(void)
 		SDL_SetError("Joystick: Couldn't create a HID object iterator.");
 		return -1;
 	}
-	if (NULL == hidObjectIterator) /* there are no joysticks */
+	if (!hidObjectIterator) /* there are no joysticks */
 	{
 		gpDeviceList = NULL;
 		SDL_numjoysticks = 0;
@@ -661,8 +645,9 @@ int SDL_SYS_JoystickInit(void)
 
 		/* dump device object, it is no longer needed */
 		result = IOObjectRelease (ioHIDDeviceObject);
-//		if (KERN_SUCCESS != result)
-//			HIDReportErrorNum ("IOObjectRelease error with ioHIDDeviceObject.", result);
+/*		if (KERN_SUCCESS != result)
+			HIDReportErrorNum ("IOObjectRelease error with ioHIDDeviceObject.", result);
+*/
 
 		/* Filter device list to non-keyboard/mouse stuff */ 
 		if ( (device->usagePage != kHIDPage_GenericDesktop) ||
@@ -853,3 +838,5 @@ void SDL_SYS_JoystickQuit(void)
 	while (NULL != gpDeviceList)
 		gpDeviceList = HIDDisposeDevice (&gpDeviceList);
 }
+
+#endif /* SDL_JOYSTICK_IOKIT */
