@@ -1,52 +1,43 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_sysjoystick.c,v 1.19 2004/09/11 17:48:41 slouken Exp $";
-#endif
+#ifdef SDL_JOYSTICK_LINUX
 
 /* This is the system specific header for the SDL joystick API */
 
-#include <stdio.h>		/* For the definition of NULL */
-#include <stdlib.h>		/* For getenv() prototype */
-#include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <limits.h>		/* For the definition of PATH_MAX */
-#ifdef __arm__
-#include <linux/limits.h> /* Arm cross-compiler needs this */
-#endif
 #include <linux/joystick.h>
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 #include <linux/input.h>
 #endif
 
-#include "SDL_error.h"
 #include "SDL_joystick.h"
-#include "SDL_sysjoystick.h"
-#include "SDL_joystick_c.h"
+#include "../SDL_sysjoystick.h"
+#include "../SDL_joystick_c.h"
 
 /* Special joystick configurations */
 static struct {
@@ -69,45 +60,98 @@ static struct {
 
 #ifndef NO_LOGICAL_JOYSTICKS
 
-static struct joystick_logical_values {
+/*
+   Some USB HIDs show up as a single joystick even though they actually
+   control 2 or more joysticks.
+*/
+/*
+   This code handles the MP-8800 (Quad) and MP-8866 (Dual), which can
+   be identified by their transparent blue design. It's quite trivial
+   to add other joysticks with similar quirky behavior.
+   -id
+*/
+
+struct joystick_logical_mapping {
         int njoy;
         int nthing;
-} joystick_logical_values[] = {
+};
 
-/* +0 */
-   /* MP-8800 axes map - map to {logical joystick #, logical axis #} */
-   {0,0},{0,1},{0,2},{1,0},{1,1},{0,3},{1,2},{1,3},{2,0},{2,1},{2,2},{2,3},
-   {3,0},{3,1},{3,2},{3,3},{0,4},{1,4},{2,4},
+/*
+   {logical joy, logical axis},
+   {logical joy, logical hat},
+   {logical joy, logical ball},
+   {logical joy, logical button}
+*/
 
-/* +19 */
-   /* MP-8800 hat map - map to {logical joystick #, logical hat #} */
-   {0,0},{1,0},{2,0},{3,0},
+static struct joystick_logical_mapping mp88xx_1_logical_axismap[] = {
+   {0,0},{0,1},{0,2},{0,3},{0,4},{0,5}
+};
+static struct joystick_logical_mapping mp88xx_1_logical_buttonmap[] = {
+   {0,0},{0,1},{0,2},{0,3},{0,4},{0,5},{0,6},{0,7},{0,8},{0,9},{0,10},{0,11}
+};
 
-/* +23 */
-   /* MP-8800 button map - map to {logical joystick #, logical button #} */
+static struct joystick_logical_mapping mp88xx_2_logical_axismap[] = {
+   {0,0},{0,1},{0,2},{1,0},{1,1},{0,3},
+   {1,2},{1,3},{0,4},{0,5},{1,4},{1,5}
+};
+static struct joystick_logical_mapping mp88xx_2_logical_buttonmap[] = {
+   {0,0},{0,1},{0,2},{0,3},{0,4},{0,5},{0,6},{0,7},{0,8},{0,9},{0,10},{0,11},
+   {1,0},{1,1},{1,2},{1,3},{1,4},{1,5},{1,6},{1,7},{1,8},{1,9},{1,10},{1,11}
+};
+
+static struct joystick_logical_mapping mp88xx_3_logical_axismap[] = {
+   {0,0},{0,1},{0,2},{1,0},{1,1},{0,3},
+   {1,2},{1,3},{2,0},{2,1},{2,2},{2,3},
+   {0,4},{0,5},{1,4},{1,5},{2,4},{2,5}
+};
+static struct joystick_logical_mapping mp88xx_3_logical_buttonmap[] = {
+   {0,0},{0,1},{0,2},{0,3},{0,4},{0,5},{0,6},{0,7},{0,8},{0,9},{0,10},{0,11},
+   {1,0},{1,1},{1,2},{1,3},{1,4},{1,5},{1,6},{1,7},{1,8},{1,9},{1,10},{1,11},
+   {2,0},{2,1},{2,2},{2,3},{2,4},{2,5},{2,6},{2,7},{2,8},{2,9},{2,10},{2,11}
+};
+
+static struct joystick_logical_mapping mp88xx_4_logical_axismap[] = {
+   {0,0},{0,1},{0,2},{1,0},{1,1},{0,3},
+   {1,2},{1,3},{2,0},{2,1},{2,2},{2,3},
+   {3,0},{3,1},{3,2},{3,3},{0,4},{0,5},
+   {1,4},{1,5},{2,4},{2,5},{3,4},{3,5}
+};
+static struct joystick_logical_mapping mp88xx_4_logical_buttonmap[] = {
    {0,0},{0,1},{0,2},{0,3},{0,4},{0,5},{0,6},{0,7},{0,8},{0,9},{0,10},{0,11},
    {1,0},{1,1},{1,2},{1,3},{1,4},{1,5},{1,6},{1,7},{1,8},{1,9},{1,10},{1,11},
    {2,0},{2,1},{2,2},{2,3},{2,4},{2,5},{2,6},{2,7},{2,8},{2,9},{2,10},{2,11},
    {3,0},{3,1},{3,2},{3,3},{3,4},{3,5},{3,6},{3,7},{3,8},{3,9},{3,10},{3,11}
 };
 
-static struct joystick_logical_layout {
+struct joystick_logical_layout {
         int naxes;
         int nhats;
         int nballs;
         int nbuttons;
-} joystick_logical_layout[] = {
-        /* MP-8800 logical layout */
-        {5, 1, 0, 12},
-        {5, 1, 0, 12},
-        {5, 1, 0, 12},
-        {4, 1, 0, 12}
+};
+
+static struct joystick_logical_layout mp88xx_1_logical_layout[] = {
+        {6, 0, 0, 12}
+};
+static struct joystick_logical_layout mp88xx_2_logical_layout[] = {
+        {6, 0, 0, 12},
+        {6, 0, 0, 12}
+};
+static struct joystick_logical_layout mp88xx_3_logical_layout[] = {
+        {6, 0, 0, 12},
+        {6, 0, 0, 12},
+        {6, 0, 0, 12}
+};
+static struct joystick_logical_layout mp88xx_4_logical_layout[] = {
+        {6, 0, 0, 12},
+        {6, 0, 0, 12},
+        {6, 0, 0, 12},
+        {6, 0, 0, 12}
 };
 
 /*
-   Some USB HIDs show up as a single joystick even though they actually
-   control 2 or more joysticks.  This array sets up a means of mapping
-   a single physical joystick to multiple logical joysticks. (djm)
+   This array sets up a means of mapping a single physical joystick to
+   multiple logical joysticks. (djm)
                                                                                 
    njoys
         the number of logical joysticks
@@ -118,19 +162,78 @@ static struct joystick_logical_layout {
    axes, hats, balls, buttons
         arrays that map a physical thingy to a logical thingy
  */
-static struct joystick_logicalmap {
+struct joystick_logicalmap {
         const char *name;
+	int nbuttons;
         int njoys;
-        struct joystick_logical_layout *layouts;
-        struct joystick_logical_values *axes;
-        struct joystick_logical_values *hats;
-        struct joystick_logical_values *balls;
-        struct joystick_logical_values *buttons;
+        struct joystick_logical_layout *layout;
+        struct joystick_logical_mapping *axismap;
+        struct joystick_logical_mapping *hatmap;
+        struct joystick_logical_mapping *ballmap;
+        struct joystick_logical_mapping *buttonmap;
+};
 
-} joystick_logicalmap[] = {
-        {"WiseGroup.,Ltd MP-8800 Quad USB Joypad", 4, joystick_logical_layout,
-         joystick_logical_values, joystick_logical_values+19, NULL,
-         joystick_logical_values+23}
+static struct joystick_logicalmap joystick_logicalmap[] = {
+        {
+		"WiseGroup.,Ltd MP-8866 Dual USB Joypad",
+		12,
+		1,
+		mp88xx_1_logical_layout,
+        	mp88xx_1_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_1_logical_buttonmap
+	},
+        {
+		"WiseGroup.,Ltd MP-8866 Dual USB Joypad",
+		24,
+		2,
+		mp88xx_2_logical_layout,
+        	mp88xx_2_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_2_logical_buttonmap
+	},
+        {
+		"WiseGroup.,Ltd MP-8800 Quad USB Joypad",
+		12,
+		1,
+		mp88xx_1_logical_layout,
+        	mp88xx_1_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_1_logical_buttonmap
+	},
+        {
+		"WiseGroup.,Ltd MP-8800 Quad USB Joypad",
+		24,
+		2,
+		mp88xx_2_logical_layout,
+        	mp88xx_2_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_2_logical_buttonmap
+	},
+        {
+		"WiseGroup.,Ltd MP-8800 Quad USB Joypad",
+		36,
+		3,
+		mp88xx_3_logical_layout,
+        	mp88xx_3_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_3_logical_buttonmap
+	},
+        {
+		"WiseGroup.,Ltd MP-8800 Quad USB Joypad",
+		48,
+		4,
+		mp88xx_4_logical_layout,
+        	mp88xx_4_logical_axismap,
+		NULL,
+		NULL,
+        	mp88xx_4_logical_buttonmap
+	}
 };
 
 /* find the head of a linked list, given a point in it
@@ -177,7 +280,7 @@ struct joystick_hwdata {
 	} *balls;
 
 	/* Support for the Linux 2.4 unified input interface */
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 	SDL_bool is_hid;
 	Uint8 key_map[KEY_MAX-BTN_MISC];
 	Uint8 abs_map[ABS_MAX];
@@ -188,17 +291,6 @@ struct joystick_hwdata {
 #endif
 };
 
-static char *mystrdup(const char *string)
-{
-	char *newstring;
-
-	newstring = (char *)malloc(strlen(string)+1);
-	if ( newstring ) {
-		strcpy(newstring, string);
-	}
-	return(newstring);
-}
-
 
 #ifndef NO_LOGICAL_JOYSTICKS
 
@@ -206,28 +298,40 @@ static int CountLogicalJoysticks(int max)
 {
    register int i, j, k, ret, prev;
    const char* name;
+   int nbuttons, fd;
+   unsigned char n;
 
    ret = 0;
 
    for(i = 0; i < max; i++) {
       name = SDL_SYS_JoystickName(i);
+	
+      fd = open(SDL_joylist[i].fname, O_RDONLY, 0);
+      if ( fd >= 0 ) {
+	 if ( ioctl(fd, JSIOCGBUTTONS, &n) < 0 ) {
+	    nbuttons = -1;
+	 } else {
+            nbuttons = n;
+	 }
+	 close(fd);
+      }
+      else {
+	 nbuttons=-1;
+      }
 
       if (name) {
-         for(j = 0; j < SDL_TABLESIZE(joystick_logicalmap); j++) {
-            if (!strcmp(name, joystick_logicalmap[j].name)) {
-
+         for(j = 0; j < SDL_arraysize(joystick_logicalmap); j++) {
+            if (!SDL_strcmp(name, joystick_logicalmap[j].name) && (nbuttons==-1 || nbuttons==joystick_logicalmap[j].nbuttons)) {
                prev = i;
-               SDL_joylist[prev].map = joystick_logicalmap+j;
+               SDL_joylist[prev].map = &(joystick_logicalmap[j]);
 
                for(k = 1; k < joystick_logicalmap[j].njoys; k++) {
                   SDL_joylist[prev].next = max + ret;
-
-                  if (prev != i)
-                     SDL_joylist[max+ret].prev = prev;
-
+                  SDL_joylist[max+ret].prev = prev;
+		  
                   prev = max + ret;
                   SDL_joylist[prev].logicalno = k;
-                  SDL_joylist[prev].map = joystick_logicalmap+j;
+                  SDL_joylist[prev].map = &(joystick_logicalmap[j]);
                   ret++;
                }
 
@@ -247,9 +351,7 @@ static void LogicalSuffix(int logicalno, char* namebuf, int len)
       "01020304050607080910111213141516171819"
       "20212223242526272829303132";
    const char* suffix;
-
-   slen = strlen(namebuf);
-
+   slen = SDL_strlen(namebuf);
    suffix = NULL;
 
    if (logicalno*2<sizeof(suffixs))
@@ -266,7 +368,7 @@ static void LogicalSuffix(int logicalno, char* namebuf, int len)
 
 #endif /* USE_LOGICAL_JOYSTICKS */
 
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 #define test_bit(nr, addr) \
 	(((1UL << ((nr) & 31)) & (((const unsigned int *) addr)[(nr) >> 5])) != 0)
 
@@ -287,14 +389,14 @@ static int EV_IsJoystick(int fd)
 	return(1);
 }
 
-#endif /* USE_INPUT_EVENTS */
+#endif /* SDL_INPUT_LINUXEV */
 
 /* Function to scan the system for joysticks */
 int SDL_SYS_JoystickInit(void)
 {
 	/* The base path of the joystick devices */
 	const char *joydev_pattern[] = {
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 		"/dev/input/event%d",
 #endif
 		"/dev/input/js%d",
@@ -311,14 +413,13 @@ int SDL_SYS_JoystickInit(void)
 	numjoysticks = 0;
 
 	/* First see if the user specified a joystick to use */
-	if ( getenv("SDL_JOYSTICK_DEVICE") != NULL ) {
-		strncpy(path, getenv("SDL_JOYSTICK_DEVICE"), sizeof(path));
-		path[sizeof(path)-1] = '\0';
+	if ( SDL_getenv("SDL_JOYSTICK_DEVICE") != NULL ) {
+		SDL_strlcpy(path, SDL_getenv("SDL_JOYSTICK_DEVICE"), sizeof(path));
 		if ( stat(path, &sb) == 0 ) {
 			fd = open(path, O_RDONLY, 0);
 			if ( fd >= 0 ) {
 				/* Assume the user knows what they're doing. */
-				SDL_joylist[numjoysticks].fname =mystrdup(path);
+				SDL_joylist[numjoysticks].fname = SDL_strdup(path);
 				if ( SDL_joylist[numjoysticks].fname ) {
 					dev_nums[numjoysticks] = sb.st_rdev;
 					++numjoysticks;
@@ -328,9 +429,9 @@ int SDL_SYS_JoystickInit(void)
 		}
 	}
 
-	for ( i=0; i<SDL_TABLESIZE(joydev_pattern); ++i ) {
+	for ( i=0; i<SDL_arraysize(joydev_pattern); ++i ) {
 		for ( j=0; j < MAX_JOYSTICKS; ++j ) {
-			sprintf(path, joydev_pattern[i], j);
+			SDL_snprintf(path, SDL_arraysize(path), joydev_pattern[i], j);
 
 			/* rcg06302000 replaced access(F_OK) call with stat().
 			 * stat() will fail if the file doesn't exist, so it's
@@ -354,7 +455,7 @@ int SDL_SYS_JoystickInit(void)
 				if ( fd < 0 ) {
 					continue;
 				}
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 #ifdef DEBUG_INPUT_EVENTS
 				printf("Checking %s\n", path);
 #endif
@@ -366,7 +467,7 @@ int SDL_SYS_JoystickInit(void)
 				close(fd);
 
 				/* We're fine, add this joystick */
-				SDL_joylist[numjoysticks].fname =mystrdup(path);
+				SDL_joylist[numjoysticks].fname = SDL_strdup(path);
 				if ( SDL_joylist[numjoysticks].fname ) {
 					dev_nums[numjoysticks] = sb.st_rdev;
 					++numjoysticks;
@@ -375,7 +476,7 @@ int SDL_SYS_JoystickInit(void)
 				break;
 		}
 
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 		/* This is a special case...
 		   If the event devices are valid then the joystick devices
 		   will be duplicates but without extra information about their
@@ -409,7 +510,7 @@ const char *SDL_SYS_JoystickName(int index)
 	fd = open(SDL_joylist[index].fname, O_RDONLY, 0);
 	if ( fd >= 0 ) {
 		if ( 
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 		     (ioctl(fd, EVIOCGNAME(sizeof(namebuf)), namebuf) <= 0) &&
 #endif
 		     (ioctl(fd, JSIOCGNAME(sizeof(namebuf)), namebuf) <= 0) ) {
@@ -419,9 +520,12 @@ const char *SDL_SYS_JoystickName(int index)
 		}
 		close(fd);
 
+
 #ifndef NO_LOGICAL_JOYSTICKS
-		if (SDL_joylist[oindex].prev || SDL_joylist[oindex].next)
+		if (SDL_joylist[oindex].prev || SDL_joylist[oindex].next || index!=oindex)
+		{
        		   LogicalSuffix(SDL_joylist[oindex].logicalno, namebuf, 128);
+		}
 #endif
 	}
 	return name;
@@ -431,7 +535,7 @@ static int allocate_hatdata(SDL_Joystick *joystick)
 {
 	int i;
 
-	joystick->hwdata->hats = (struct hwdata_hat *)malloc(
+	joystick->hwdata->hats = (struct hwdata_hat *)SDL_malloc(
 		joystick->nhats * sizeof(struct hwdata_hat));
 	if ( joystick->hwdata->hats == NULL ) {
 		return(-1);
@@ -447,7 +551,7 @@ static int allocate_balldata(SDL_Joystick *joystick)
 {
 	int i;
 
-	joystick->hwdata->balls = (struct hwdata_ball *)malloc(
+	joystick->hwdata->balls = (struct hwdata_ball *)SDL_malloc(
 		joystick->nballs * sizeof(struct hwdata_ball));
 	if ( joystick->hwdata->balls == NULL ) {
 		return(-1);
@@ -486,8 +590,8 @@ static SDL_bool JS_ConfigJoystick(SDL_Joystick *joystick, int fd)
 	old_axes = joystick->naxes;
 
 	/* Generic analog joystick support */
-	if ( strstr(name, "Analog") == name && strstr(name, "-hat") ) {
-		if ( sscanf(name,"Analog %d-axis %*d-button %d-hat",
+	if ( SDL_strstr(name, "Analog") == name && SDL_strstr(name, "-hat") ) {
+		if ( SDL_sscanf(name,"Analog %d-axis %*d-button %d-hat",
 			&tmp_naxes, &tmp_nhats) == 2 ) {
 
 			joystick->naxes = tmp_naxes;
@@ -498,8 +602,8 @@ static SDL_bool JS_ConfigJoystick(SDL_Joystick *joystick, int fd)
 	}
 
 	/* Special joystick support */
-	for ( i=0; i < SDL_TABLESIZE(special_joysticks); ++i ) {
-		if ( strcmp(name, special_joysticks[i].name) == 0 ) {
+	for ( i=0; i < SDL_arraysize(special_joysticks); ++i ) {
+		if ( SDL_strcmp(name, special_joysticks[i].name) == 0 ) {
 
 			joystick->naxes = special_joysticks[i].naxes;
 			joystick->nhats = special_joysticks[i].nhats;
@@ -511,16 +615,16 @@ static SDL_bool JS_ConfigJoystick(SDL_Joystick *joystick, int fd)
 	}
 
 	/* User environment joystick support */
-	if ( (env = getenv("SDL_LINUX_JOYSTICK")) ) {
-		strcpy(env_name, "");
-		if ( *env == '\'' && sscanf(env, "'%[^']s'", env_name) == 1 )
-			env += strlen(env_name)+2;
-		else if ( sscanf(env, "%s", env_name) == 1 )
-			env += strlen(env_name);
+	if ( (env = SDL_getenv("SDL_LINUX_JOYSTICK")) ) {
+		*env_name = '\0';
+		if ( *env == '\'' && SDL_sscanf(env, "'%[^']s'", env_name) == 1 )
+			env += SDL_strlen(env_name)+2;
+		else if ( SDL_sscanf(env, "%s", env_name) == 1 )
+			env += SDL_strlen(env_name);
 
-		if ( strcmp(name, env_name) == 0 ) {
+		if ( SDL_strcmp(name, env_name) == 0 ) {
 
-			if ( sscanf(env, "%d %d %d", &tmp_naxes, &tmp_nhats,
+			if ( SDL_sscanf(env, "%d %d %d", &tmp_naxes, &tmp_nhats,
 				&tmp_nballs) == 3 ) {
 
 				joystick->naxes = tmp_naxes;
@@ -549,7 +653,7 @@ static SDL_bool JS_ConfigJoystick(SDL_Joystick *joystick, int fd)
 	return(handled);
 }
 
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 
 static SDL_bool EV_ConfigJoystick(SDL_Joystick *joystick, int fd)
 {
@@ -648,14 +752,14 @@ static SDL_bool EV_ConfigJoystick(SDL_Joystick *joystick, int fd)
 	return(joystick->hwdata->is_hid);
 }
 
-#endif /* USE_INPUT_EVENTS */
+#endif /* SDL_INPUT_LINUXEV */
 
 #ifndef NO_LOGICAL_JOYSTICKS
 static void ConfigLogicalJoystick(SDL_Joystick *joystick)
 {
         struct joystick_logical_layout* layout;
                                                                                 
-        layout = SDL_joylist[joystick->index].map->layouts +
+        layout = SDL_joylist[joystick->index].map->layout +
                 SDL_joylist[joystick->index].logicalno;
                                                                                 
         joystick->nbuttons = layout->nbuttons;
@@ -702,13 +806,13 @@ int SDL_SYS_JoystickOpen(SDL_Joystick *joystick)
 		return(-1);
 	}
 	joystick->hwdata = (struct joystick_hwdata *)
-	                   malloc(sizeof(*joystick->hwdata));
+	                   SDL_malloc(sizeof(*joystick->hwdata));
 	if ( joystick->hwdata == NULL ) {
 		SDL_OutOfMemory();
 		close(fd);
 		return(-1);
 	}
-	memset(joystick->hwdata, 0, sizeof(*joystick->hwdata));
+	SDL_memset(joystick->hwdata, 0, sizeof(*joystick->hwdata));
 	joystick->hwdata->fd = fd;
 
 	/* Set the joystick to non-blocking read mode */
@@ -720,7 +824,7 @@ int SDL_SYS_JoystickOpen(SDL_Joystick *joystick)
 		ConfigLogicalJoystick(joystick);
 	else
 #endif
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 	if ( ! EV_ConfigJoystick(joystick, fd) )
 #endif
 		JS_ConfigJoystick(joystick, fd);
@@ -731,7 +835,7 @@ int SDL_SYS_JoystickOpen(SDL_Joystick *joystick)
 #ifndef NO_LOGICAL_JOYSTICKS
 
 static SDL_Joystick* FindLogicalJoystick(
-   SDL_Joystick *joystick, struct joystick_logical_values* v)
+   SDL_Joystick *joystick, struct joystick_logical_mapping* v)
 {
         SDL_Joystick *logicaljoy;
         register int i;
@@ -760,7 +864,7 @@ static SDL_Joystick* FindLogicalJoystick(
 
 static int LogicalJoystickButton(
    SDL_Joystick *joystick, Uint8 button, Uint8 state){
-        struct joystick_logical_values* buttons;
+        struct joystick_logical_mapping* buttons;
         SDL_Joystick *logicaljoy = NULL;
 
         /* if there's no map then this is just a regular joystick
@@ -770,7 +874,7 @@ static int LogicalJoystickButton(
 
         /* get the logical joystick that will receive the event
          */
-        buttons = SDL_joylist[joystick->index].map->buttons+button;
+        buttons = SDL_joylist[joystick->index].map->buttonmap+button;
         logicaljoy = FindLogicalJoystick(joystick, buttons);
 
         if (logicaljoy == NULL)
@@ -784,7 +888,7 @@ static int LogicalJoystickButton(
 static int LogicalJoystickAxis(
 	SDL_Joystick *joystick, Uint8 axis, Sint16 value)
 {
-        struct joystick_logical_values* axes;
+        struct joystick_logical_mapping* axes;
         SDL_Joystick *logicaljoy = NULL;
 
         /* if there's no map then this is just a regular joystick
@@ -794,7 +898,7 @@ static int LogicalJoystickAxis(
 
         /* get the logical joystick that will receive the event
          */
-        axes = SDL_joylist[joystick->index].map->axes+axis;
+        axes = SDL_joylist[joystick->index].map->axismap+axis;
         logicaljoy = FindLogicalJoystick(joystick, axes);
 
         if (logicaljoy == NULL)
@@ -816,7 +920,7 @@ void HandleHat(SDL_Joystick *stick, Uint8 hat, int axis, int value)
 		{ SDL_HAT_LEFTDOWN, SDL_HAT_DOWN, SDL_HAT_RIGHTDOWN }
 	};
 	SDL_logical_joydecl(SDL_Joystick *logicaljoy = NULL);
-	SDL_logical_joydecl(struct joystick_logical_values* hats = NULL);
+	SDL_logical_joydecl(struct joystick_logical_mapping* hats = NULL);
 
 	the_hat = &stick->hwdata->hats[hat];
 	if ( value < 0 ) {
@@ -838,7 +942,7 @@ void HandleHat(SDL_Joystick *stick, Uint8 hat, int axis, int value)
 
 			/* get the fake joystick that will receive the event
 			*/
-			hats = SDL_joylist[stick->index].map->hats+hat;
+			hats = SDL_joylist[stick->index].map->hatmap+hat;
 			logicaljoy = FindLogicalJoystick(stick, hats);
 		}
 
@@ -873,7 +977,8 @@ static __inline__ void JS_HandleEvents(SDL_Joystick *joystick)
 #ifndef NO_LOGICAL_JOYSTICKS
 	if (SDL_joylist[joystick->index].fname == NULL) {
 		SDL_joylist_head(i, joystick->index);
-		return JS_HandleEvents(SDL_joylist[i].joy);
+		JS_HandleEvents(SDL_joylist[i].joy);
+		return;
 	}
 #endif
 
@@ -923,7 +1028,7 @@ static __inline__ void JS_HandleEvents(SDL_Joystick *joystick)
 		}
 	}
 }
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 static __inline__ int EV_AxisCorrect(SDL_Joystick *joystick, int which, int value)
 {
 	struct axis_correct *correct;
@@ -943,7 +1048,7 @@ static __inline__ int EV_AxisCorrect(SDL_Joystick *joystick, int which, int valu
 	}
 
 	/* Clamp and return */
-	if ( value < -32767 ) return -32767;
+	if ( value < -32768 ) return -32768;
 	if ( value >  32767 ) return  32767;
 
 	return value;
@@ -1025,13 +1130,13 @@ static __inline__ void EV_HandleEvents(SDL_Joystick *joystick)
 		}
 	}
 }
-#endif /* USE_INPUT_EVENTS */
+#endif /* SDL_INPUT_LINUXEV */
 
 void SDL_SYS_JoystickUpdate(SDL_Joystick *joystick)
 {
 	int i;
 	
-#ifdef USE_INPUT_EVENTS
+#if SDL_INPUT_LINUXEV
 	if ( joystick->hwdata->is_hid )
 		EV_HandleEvents(joystick);
 	else
@@ -1069,12 +1174,12 @@ void SDL_SYS_JoystickClose(SDL_Joystick *joystick)
 #endif
 		close(joystick->hwdata->fd);
 		if ( joystick->hwdata->hats ) {
-			free(joystick->hwdata->hats);
+			SDL_free(joystick->hwdata->hats);
 		}
 		if ( joystick->hwdata->balls ) {
-			free(joystick->hwdata->balls);
+			SDL_free(joystick->hwdata->balls);
 		}
-		free(joystick->hwdata);
+		SDL_free(joystick->hwdata);
 		joystick->hwdata = NULL;
 	}
 }
@@ -1085,8 +1190,9 @@ void SDL_SYS_JoystickQuit(void)
 	int i;
 
 	for ( i=0; SDL_joylist[i].fname; ++i ) {
-		free(SDL_joylist[i].fname);
+		SDL_free(SDL_joylist[i].fname);
 	}
 	SDL_joylist[0].fname = NULL;
 }
 
+#endif /* SDL_JOYSTICK_LINUX */
