@@ -1,29 +1,27 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_sysjoystick.c,v 1.8 2005/06/25 18:21:13 pmandin Exp $";
-#endif
+#ifdef SDL_JOYSTICK_MINT
 
 /*
  *	Atari Joystick/Joypad drivers
@@ -31,22 +29,16 @@ static char rcsid =
  *	Patrice Mandin
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include <mint/cookie.h>
 #include <mint/osbind.h>
 
-#include "SDL_types.h"
-#include "SDL_error.h"
-#include "SDL_joystick.h"
-#include "SDL_sysjoystick.h"
-#include "SDL_joystick_c.h"
+#include "SDL_events.h"
+#include "../SDL_sysjoystick.h"
+#include "../SDL_joystick_c.h"
 
-#include "SDL_ikbdinterrupt_s.h"
-#include "SDL_xbiosevents_c.h"
-#include "SDL_xbiosinterrupt_s.h"
+#include "../../video/ataricommon/SDL_ikbdinterrupt_s.h"
+#include "../../video/ataricommon/SDL_xbiosevents_c.h"
+#include "../../video/ataricommon/SDL_xbiosinterrupt_s.h"
 
 /*--- Const ---*/
 
@@ -167,13 +159,13 @@ int SDL_SYS_JoystickInit(void)
 {
 	int i;
 	unsigned long cookie_mch;
-	const char *envr=getenv("SDL_JOYSTICK_ATARI");
+	const char *envr=SDL_getenv("SDL_JOYSTICK_ATARI");
 	
 #define TEST_JOY_ENABLED(env,idstring,num) \
-	if (strstr(env,idstring"-off")) { \
+	if (SDL_strstr(env,idstring"-off")) { \
 		atarijoysticks[num].enabled=SDL_FALSE; \
 	} \
-	if (strstr(env,idstring"-on")) { \
+	if (SDL_strstr(env,idstring"-on")) { \
 		atarijoysticks[num].enabled=SDL_TRUE; \
 	}
 
@@ -637,3 +629,5 @@ static void UpdateJoypads(void)
 	JOYPAD_IO.directions=0xff7f;
 	jp_joypads[1] |= (((~(JOYPAD_IO.directions))>>12) & 15)<<12;
 }
+
+#endif /* SDL_JOYSTICK_MINT */
