@@ -1,48 +1,45 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
 
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_BWin.h,v 1.8 2004/12/14 18:23:27 slouken Exp $";
-#endif
-
 #ifndef _SDL_BWin_h
 #define _SDL_BWin_h
 
+#include "SDL_config.h"
+
 #include <stdio.h>
 #include <AppKit.h>
 #include <InterfaceKit.h>
 #include <be/game/DirectWindow.h>
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 #include <be/opengl/GLView.h>
 #endif
 #include <support/UTF8.h>
 
-#include "SDL_BeApp.h"
+#include "../../main/beos/SDL_BeApp.h"
 #include "SDL_events.h"
 #include "SDL_BView.h"
 
 extern "C" {
-#include "SDL_events_c.h"
+#include "../../events/SDL_events_c.h"
 };
 
 class SDL_BWin : public BDirectWindow
@@ -54,7 +51,7 @@ public:
 		last_buttons = 0;
 
 		the_view = NULL;
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 		SDL_GLView = NULL;
 #endif
 		SDL_View = NULL;
@@ -66,7 +63,7 @@ public:
 	virtual ~SDL_BWin() {
 		Lock();
 		if ( the_view ) {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 			if ( the_view == SDL_GLView ) {
 				SDL_GLView->UnlockGL();
 			}
@@ -75,7 +72,7 @@ public:
 			the_view = NULL;
 		}
 		Unlock();
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 		if ( SDL_GLView ) {
 			delete SDL_GLView;
 		}
@@ -223,7 +220,7 @@ public:
 		retval = 0;
 		Lock();
 		if ( flags & SDL_OPENGL ) {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 			if ( SDL_GLView == NULL ) {
 				SDL_GLView = new BGLView(Bounds(), "SDL GLView",
 					 	B_FOLLOW_ALL_SIDES, (B_WILL_DRAW|B_FRAME_EVENTS),
@@ -247,7 +244,7 @@ public:
 			}
 			if ( the_view != SDL_View ) {
 				if ( the_view ) {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 					if ( the_view == SDL_GLView ) {
 						SDL_GLView->UnlockGL();
 					}
@@ -265,7 +262,7 @@ public:
 		SDL_View->SetBitmap(bitmap);
 	}
 	virtual void SetXYOffset(int x, int y) {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 		if ( the_view == SDL_GLView ) {
 			return;
 		}
@@ -273,7 +270,7 @@ public:
 		SDL_View->SetXYOffset(x, y);		
 	}
 	virtual void GetXYOffset(int &x, int &y) {
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 		if ( the_view == SDL_GLView ) {
 			x = 0;
 			y = 0;
@@ -292,7 +289,7 @@ public:
 		SDL_View->Sync();
 		Unlock();
 	}
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	virtual void SwapBuffers(void) {
 		SDL_GLView->UnlockGL();
 		SDL_GLView->LockGL();
@@ -336,7 +333,7 @@ public:
 
 		if ((uchar)buf[0] > 127) {
 			state = 0;
-			srclen = strlen(buf);
+			srclen = SDL_strlen(buf);
 			dstlen = sizeof(destbuf);
 			convert_from_utf8(B_UNICODE_CONVERSION, buf, &srclen, (char *)destbuf, &dstlen, &state);
 			unicode = destbuf[0];
@@ -481,7 +478,7 @@ public:
 				if (msg->FindInt32("key", &key) == B_OK && msg->FindInt32("modifiers", &modifiers) == B_OK) {
 					SDL_keysym keysym;
 					keysym.scancode = key;
-					if (key < 128) {
+					if ((key > 0) && (key < 128)) {
 						keysym.sym = keymap[key];
 					} else {
 						keysym.sym = SDLK_UNKNOWN;
@@ -514,7 +511,7 @@ public:
 				if (msg->FindInt32("key", &key) == B_OK && msg->FindInt32("modifiers", &modifiers) == B_OK) {
 					SDL_keysym keysym;
 					keysym.scancode = key;
-					if (key < 128) {
+					if ((key > 0) && (key < 128)) {
 						keysym.sym = keymap[key];
 					} else {
 						keysym.sym = SDLK_UNKNOWN;
@@ -545,7 +542,7 @@ public:
 	}
 
 private:
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	BGLView *SDL_GLView;
 #endif
 	SDL_BView *SDL_View;
