@@ -1,54 +1,44 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_sysvideo.cc,v 1.10 2004/07/18 19:14:33 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 /* BWindow based framebuffer implementation */
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <stdio.h>
 #include <unistd.h>
 
-#include "SDL.h"
-#include "SDL_BeApp.h"
 #include "SDL_BWin.h"
 #include "SDL_timer.h"
-#include "blank_cursor.h"
 
 extern "C" {
 
-#include "SDL_sysvideo.h"
-#include "SDL_sysmouse_c.h"
+#include "../SDL_sysvideo.h"
+#include "../../events/SDL_events_c.h"
 #include "SDL_sysevents_c.h"
-#include "SDL_events_c.h"
+#include "SDL_sysmouse_c.h"
 #include "SDL_syswm_c.h"
 #include "SDL_lowvideo.h"
-#include "SDL_yuvfuncs.h"
+#include "../SDL_yuvfuncs.h"
 #include "SDL_sysyuv.h"
+#include "../blank_cursor.h"
 
 #define BEOS_HIDDEN_SIZE	32	/* starting hidden window size */
 
@@ -70,7 +60,7 @@ static int BE_ToggleFullScreen(_THIS, int fullscreen);
 static SDL_Overlay *BE_CreateYUVOverlay(_THIS, int width, int height, Uint32 format, SDL_Surface *display);
 
 /* OpenGL functions */
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 static int BE_GL_LoadLibrary(_THIS, const char *path);
 static void* BE_GL_GetProcAddress(_THIS, const char *proc);
 static int BE_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value);
@@ -87,8 +77,8 @@ static int BE_Available(void)
 
 static void BE_DeleteDevice(SDL_VideoDevice *device)
 {
-	free(device->hidden);
-	free(device);
+	SDL_free(device->hidden);
+	SDL_free(device);
 }
 
 static SDL_VideoDevice *BE_CreateDevice(int devindex)
@@ -96,20 +86,20 @@ static SDL_VideoDevice *BE_CreateDevice(int devindex)
 	SDL_VideoDevice *device;
 
 	/* Initialize all variables that we clean on shutdown */
-	device = (SDL_VideoDevice *)malloc(sizeof(SDL_VideoDevice));
+	device = (SDL_VideoDevice *)SDL_malloc(sizeof(SDL_VideoDevice));
 	if ( device ) {
-		memset(device, 0, (sizeof *device));
+		SDL_memset(device, 0, (sizeof *device));
 		device->hidden = (struct SDL_PrivateVideoData *)
-				malloc((sizeof *device->hidden));
+				SDL_malloc((sizeof *device->hidden));
 	}
 	if ( (device == NULL) || (device->hidden == NULL) ) {
 		SDL_OutOfMemory();
 		if ( device ) {
-			free(device);
+			SDL_free(device);
 		}
 		return(0);
 	}
-	memset(device->hidden, 0, (sizeof *device->hidden));
+	SDL_memset(device->hidden, 0, (sizeof *device->hidden));
 
 	/* Set the function pointers */
 	/* Initialization/Query functions */
@@ -133,7 +123,7 @@ static SDL_VideoDevice *BE_CreateDevice(int devindex)
 	device->FlipHWSurface = NULL;
 	device->FreeHWSurface = BE_FreeHWSurface;
 	/* Gamma support */
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	/* OpenGL support */
 	device->GL_LoadLibrary = BE_GL_LoadLibrary;
 	device->GL_GetProcAddress = BE_GL_GetProcAddress;
@@ -238,7 +228,7 @@ static int BE_AddMode(_THIS, int index, unsigned int w, unsigned int h)
 	}
 
 	/* Set up the new video mode rectangle */
-	mode = (SDL_Rect *)malloc(sizeof *mode);
+	mode = (SDL_Rect *)SDL_malloc(sizeof *mode);
 	if ( mode == NULL ) {
 		SDL_OutOfMemory();
 		return(-1);
@@ -254,11 +244,11 @@ static int BE_AddMode(_THIS, int index, unsigned int w, unsigned int h)
 	/* Allocate the new list of modes, and fill in the new mode */
 	next_mode = SDL_nummodes[index];
 	SDL_modelist[index] = (SDL_Rect **)
-	       realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
+	       SDL_realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
 	if ( SDL_modelist[index] == NULL ) {
 		SDL_OutOfMemory();
 		SDL_nummodes[index] = 0;
-		free(mode);
+		SDL_free(mode);
 		return(-1);
 	}
 	SDL_modelist[index][next_mode] = mode;
@@ -285,6 +275,8 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 
 	/* Save the current display mode */
 	bscreen.GetMode(&saved_mode);
+	_this->info.current_w = saved_mode.virtual_width;
+	_this->info.current_h = saved_mode.virtual_height;
 
 	/* Determine the screen depth */
 	vformat->BitsPerPixel = ColorSpaceToBitsPerPixel(bscreen.ColorSpace());
@@ -296,7 +288,7 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 
 	/* Get the video modes we can switch to in fullscreen mode */
 	bscreen.GetModeList(&modes, &nmodes);
-	qsort(modes, nmodes, sizeof *modes, CompareModes);
+	SDL_qsort(modes, nmodes, sizeof *modes, CompareModes);
 	for ( i=0; i<nmodes; ++i ) {
 		bpp = ColorSpaceToBitsPerPixel(modes[i].space);
 		//if ( bpp != 0 ) { // There are bugs in changing colorspace
@@ -313,7 +305,7 @@ int BE_VideoInit(_THIS, SDL_PixelFormat *vformat)
 	bounds.bottom = BEOS_HIDDEN_SIZE;
 	SDL_Win = new SDL_BWin(bounds);
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	/* testgl application doesn't load library, just tries to load symbols */
 	/* is it correct? if so we have to load library here */
 	BE_GL_LoadLibrary(_this, NULL);
@@ -634,7 +626,7 @@ static void BE_NormalUpdate(_THIS, int numrects, SDL_Rect *rects)
 	}
 }
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 /* Passing a NULL path means load pointers from the application */
 int BE_GL_LoadLibrary(_THIS, const char *path)
 {
@@ -647,7 +639,7 @@ int BE_GL_LoadLibrary(_THIS, const char *path)
 				if (get_image_symbol((image_id)cookie,"glBegin",B_SYMBOL_TYPE_ANY,&location) == B_OK) {
 					_this->gl_config.dll_handle = (void*)cookie;
 					_this->gl_config.driver_loaded = 1;
-					strncpy(_this->gl_config.driver_path, "libGL.so", sizeof(_this->gl_config.driver_path)-1);
+					SDL_strlcpy(_this->gl_config.driver_path, "libGL.so", SDL_arraysize(_this->gl_config.driver_path));
 				}
 			}
 		}
@@ -675,7 +667,7 @@ int BE_GL_LoadLibrary(_THIS, const char *path)
 
 		if ((_this->gl_config.dll_handle = (void*)load_add_on(path)) != (void*)B_ERROR) {
 			_this->gl_config.driver_loaded = 1;
-			strncpy(_this->gl_config.driver_path, path, sizeof(_this->gl_config.driver_path)-1);
+			SDL_strlcpy(_this->gl_config.driver_path, path, SDL_arraysize(_this->gl_config.driver_path));
 		}*/
 	}
 
@@ -684,7 +676,7 @@ int BE_GL_LoadLibrary(_THIS, const char *path)
 	} else {
 		_this->gl_config.dll_handle = NULL;
 		_this->gl_config.driver_loaded = 0;
-		strcpy(_this->gl_config.driver_path, "");
+		*_this->gl_config.driver_path = '\0';
 		return -1;
 	}
 }
@@ -810,9 +802,9 @@ void BE_VideoQuit(_THIS)
 	for ( i=0; i<NUM_MODELISTS; ++i ) {
 		if ( SDL_modelist[i] ) {
 			for ( j=0; SDL_modelist[i][j]; ++j ) {
-				free(SDL_modelist[i][j]);
+				SDL_free(SDL_modelist[i][j]);
 			}
-			free(SDL_modelist[i]);
+			SDL_free(SDL_modelist[i]);
 			SDL_modelist[i] = NULL;
 		}
 	}
@@ -825,7 +817,7 @@ void BE_VideoQuit(_THIS)
 		_this->screen->pixels = NULL;
 	}
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
 	if (_this->gl_config.dll_handle != NULL)
 		unload_add_on((image_id)_this->gl_config.dll_handle);
 #endif
