@@ -1,29 +1,25 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_gemevents.c,v 1.22 2005/07/07 22:51:59 pmandin Exp $";
-#endif
+#include "SDL_config.h"
 
 /*
  * GEM SDL video driver implementation
@@ -34,17 +30,16 @@ static char rcsid =
  * Olivier Landemarre, Johan Klockars, Xavier Joubert, Claude Attard
  */
 
-#include <string.h>
-
 #include <gem.h>
 
-#include "SDL.h"
-#include "SDL_sysevents.h"
-#include "SDL_events_c.h"
+#include "../../events/SDL_sysevents.h"
+#include "../../events/SDL_events_c.h"
 #include "SDL_gemvideo.h"
 #include "SDL_gemevents_c.h"
-#include "SDL_atarikeys.h"	/* for keyboard scancodes */
-#include "SDL_xbiosinterrupt_s.h"
+#include "../ataricommon/SDL_atarikeys.h"	/* for keyboard scancodes */
+#include "../ataricommon/SDL_atarievents_c.h"
+#include "../ataricommon/SDL_xbiosevents_c.h"
+#include "../ataricommon/SDL_ataridevmouse_c.h"
 
 /* Defines */
 
@@ -61,14 +56,16 @@ static SDLKey keymap[ATARIBIOS_MAXKEYS];
 
 /* Functions prototypes */
 
-static SDL_keysym *TranslateKey(int scancode, int asciicode, SDL_keysym *keysym);
+static SDL_keysym *TranslateKey(int scancode, int asciicode, SDL_keysym *keysym,
+	SDL_bool pressed);
 static int do_messages(_THIS, short *message);
 static void do_keyboard(short kc, short ks);
 static void do_mouse(_THIS, short mx, short my, short mb, short ks);
 
 /* Functions */
 
-static SDL_keysym *TranslateKey(int scancode, int asciicode, SDL_keysym *keysym)
+static SDL_keysym *TranslateKey(int scancode, int asciicode, SDL_keysym *keysym,
+	SDL_bool pressed)
 {
 	/* Set the keysym information */
 	keysym->scancode = scancode;
@@ -80,6 +77,9 @@ static SDL_keysym *TranslateKey(int scancode, int asciicode, SDL_keysym *keysym)
 
 	keysym->mod = KMOD_NONE;
 	keysym->unicode = 0;
+	if (SDL_TranslateUNICODE && pressed) {
+		keysym->unicode = SDL_AtariToUnicodeTable[asciicode];
+	}
 
 	return(keysym);
 }
@@ -88,9 +88,9 @@ void GEM_InitOSKeymap(_THIS)
 {
 	int i;
 
-	memset(gem_currentkeyboard, 0, sizeof(gem_currentkeyboard));
-	memset(gem_previouskeyboard, 0, sizeof(gem_previouskeyboard));
-	memset(gem_currentascii, 0, sizeof(gem_currentascii));
+	SDL_memset(gem_currentkeyboard, 0, sizeof(gem_currentkeyboard));
+	SDL_memset(gem_previouskeyboard, 0, sizeof(gem_previouskeyboard));
+	SDL_memset(gem_currentascii, 0, sizeof(gem_currentascii));
 
 	/* Initialize keymap */
 	for ( i=0; i<sizeof(keymap); i++ )
@@ -133,7 +133,7 @@ void GEM_PumpEvents(_THIS)
 	int i;
 	SDL_keysym	keysym;
 
-	memset(gem_currentkeyboard,0,sizeof(gem_currentkeyboard));
+	SDL_memset(gem_currentkeyboard,0,sizeof(gem_currentkeyboard));
 	prevkc = prevks = 0;
 	
 	for (;;)
@@ -186,8 +186,18 @@ void GEM_PumpEvents(_THIS)
 			if (this->input_grab == SDL_GRAB_OFF) {
 				if (SDL_GetAppState() & SDL_APPMOUSEFOCUS) {
 					SDL_PrivateAppActive(0, SDL_APPMOUSEFOCUS);
+					if (SDL_GetAppState() & SDL_APPINPUTFOCUS) {
+						graf_mouse(ARROW, NULL);
+					}
 				} else {
 					SDL_PrivateAppActive(1, SDL_APPMOUSEFOCUS);
+					if (SDL_GetAppState() & SDL_APPINPUTFOCUS) {
+						if (GEM_cursor == (void *) -1) {
+							graf_mouse(M_OFF, NULL);
+						} else if (GEM_cursor) {
+							graf_mouse(USER_DEF, GEM_cursor->mform_p);
+						}
+					}
 				}
 			}
 		}
@@ -205,14 +215,16 @@ void GEM_PumpEvents(_THIS)
 	for (i=0; i<ATARIBIOS_MAXKEYS; i++) {
 		/* Key pressed ? */
 		if (gem_currentkeyboard[i] && !gem_previouskeyboard[i])
-			SDL_PrivateKeyboard(SDL_PRESSED, TranslateKey(i, gem_currentascii[i], &keysym));
+			SDL_PrivateKeyboard(SDL_PRESSED,
+				TranslateKey(i, gem_currentascii[i], &keysym, SDL_TRUE));
 			
 		/* Key unpressed ? */
 		if (gem_previouskeyboard[i] && !gem_currentkeyboard[i])
-			SDL_PrivateKeyboard(SDL_RELEASED, TranslateKey(i, gem_currentascii[i], &keysym));
+			SDL_PrivateKeyboard(SDL_RELEASED,
+				TranslateKey(i, gem_currentascii[i], &keysym, SDL_FALSE));
 	}
 
-	memcpy(gem_previouskeyboard,gem_currentkeyboard,sizeof(gem_previouskeyboard));
+	SDL_memcpy(gem_previouskeyboard,gem_currentkeyboard,sizeof(gem_previouskeyboard));
 
 	/* Refresh window name ? */
 	if (GEM_refresh_name) {
@@ -374,11 +386,14 @@ static void do_mouse(_THIS, short mx, short my, short mb, short ks)
 	}
 
 	/* Mouse motion ? */
-	if ((prevmousex!=mx) || (prevmousey!=my)) {
-		if (GEM_mouse_relative) {
-			SDL_PrivateMouseMotion(0, 1, SDL_AtariXbios_mousex, SDL_AtariXbios_mousey);
-			SDL_AtariXbios_mousex = SDL_AtariXbios_mousey = 0;
+	if (GEM_mouse_relative) {
+		if (GEM_usedevmouse) {
+			SDL_AtariDevMouse_PostMouseEvents(this, SDL_FALSE);
 		} else {
+			SDL_AtariXbios_PostMouseEvents(this, SDL_FALSE);
+		}
+	} else {
+		if ((prevmousex!=mx) || (prevmousey!=my)) {
 			int posx, posy;
 
 			/* Give mouse position relative to window position */
