@@ -1,40 +1,36 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
-#ifdef SAVE_RCSID
-static char rcsid =
- "@(#) $Id: SDL_macwm.c,v 1.5 2004/01/04 16:49:25 slouken Exp $";
-#endif
-
-#include <stdlib.h>
-#include <string.h>
-
-#if TARGET_API_MAC_CARBON
+#if defined(__APPLE__) && defined(__MACH__)
+#include <Carbon/Carbon.h>
+#elif TARGET_API_MAC_CARBON && (UNIVERSAL_INTERFACES_VERSION > 0x0335)
 #include <Carbon.h>
 #else
 #include <Windows.h>
 #include <Strings.h>
 #endif
 
+#include "SDL_stdinc.h"
 #include "SDL_macwm_c.h"
 
 void Mac_SetCaption(_THIS, const char *title, const char *icon)
@@ -42,7 +38,7 @@ void Mac_SetCaption(_THIS, const char *title, const char *icon)
 	/* Don't convert C to P string in place, because it may be read-only */
 	Str255		ptitle; /* MJS */
 	ptitle[0] = strlen (title);
-	memcpy(ptitle+1, title, ptitle[0]); /* MJS */
+	SDL_memcpy(ptitle+1, title, ptitle[0]); /* MJS */
 	if (SDL_Window)
 		SetWTitle(SDL_Window, ptitle); /* MJS */
 }
