@@ -1,28 +1,25 @@
 /*
     SDL - Simple DirectMedia Layer
-    Copyright (C) 1997-2004 Sam Lantinga
+    Copyright (C) 1997-2006 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
+    modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+    version 2.1 of the License, or (at your option) any later version.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+    Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
     Sam Lantinga
     slouken@libsdl.org
 */
-
-#ifdef SAVE_RCSID
-    static char rcsid = "@(#) $Id: SDL_ph_gl.h,v 1.1 2004/07/18 19:46:37 slouken Exp $";
-#endif
+#include "SDL_config.h"
 
 #ifndef __SDL_PH_GL_H__
 #define __SDL_PH_GL_H__
@@ -31,7 +28,7 @@
 
 #define DEFAULT_OPENGL "/usr/lib/libGL.so"
 
-#ifdef HAVE_OPENGL
+#if SDL_VIDEO_OPENGL
     void  ph_GL_SwapBuffers(_THIS);
     int   ph_GL_GetAttribute(_THIS, SDL_GLattr attrib, int* value);
     int   ph_GL_LoadLibrary(_THIS, const char* path);
@@ -39,6 +36,6 @@
     int   ph_GL_MakeCurrent(_THIS);
 
     int   ph_SetupOpenGLContext(_THIS, int width, int height, int bpp, Uint32 flags);
-#endif /* HAVE_OPENGL */
+#endif /* SDL_VIDEO_OPENGL */
 
 #endif /* __SDL_PH_GL_H__ */
