@@ -19,12 +19,12 @@
     Sam Lantinga
     slouken@libsdl.org
 */
+#include "SDL_config.h"
 
 #include "SDL_QuartzVideo.h"
 
-#include <stdlib.h> // For getenv()
-#include <IOKit/IOMessage.h> // For wake from sleep detection
-#include <IOKit/pwr_mgt/IOPMLib.h> // For wake from sleep detection
+#include <IOKit/IOMessage.h> /* For wake from sleep detection */
+#include <IOKit/pwr_mgt/IOPMLib.h> /* For wake from sleep detection */
 #include "SDL_QuartzKeys.h"
 
 /* 
@@ -260,19 +260,14 @@ static void QZ_DoKey (_THIS, int state, NSEvent *event) {
         contains multiple characters, we'll use 0 as
         the scancode/keysym.
     */
-    chars = [ event characters ];
-    numChars = [ chars length ];
-
-    if (numChars == 1) {
-
-        key.scancode = [ event keyCode ];
-        key.sym      = keymap [ key.scancode ];
-        key.unicode  = [ chars characterAtIndex:0 ];
-        key.mod      = KMOD_NONE;
-
-        SDL_PrivateKeyboard (state, &key);
+    if (SDL_TranslateUNICODE) {
+        chars = [ event characters ];
+        numChars = [ chars length ];
+    } else {
+        numChars = 0;
     }
-    else if (numChars == 0) {
+
+    if (numChars == 0) {
       
         key.scancode = [ event keyCode ];
         key.sym      = keymap [ key.scancode ];
@@ -281,6 +276,15 @@ static void QZ_DoKey (_THIS, int state, NSEvent *event) {
 
         SDL_PrivateKeyboard (state, &key);
     }
+    else if (numChars == 1) {
+
+        key.scancode = [ event keyCode ];
+        key.sym      = keymap [ key.scancode ];
+        key.unicode  = [ chars characterAtIndex:0 ];
+        key.mod      = KMOD_NONE;
+
+        SDL_PrivateKeyboard (state, &key);
+    }
     else /* (numChars > 1) */ {
       
         int i;
@@ -295,7 +299,7 @@ static void QZ_DoKey (_THIS, int state, NSEvent *event) {
         }
     }
     
-    if (getenv ("SDL_ENABLEAPPEVENTS"))
+    if (SDL_getenv ("SDL_ENABLEAPPEVENTS"))
         [ NSApp sendEvent:event ];
 }
 
@@ -614,8 +618,10 @@ static void QZ_GetMouseLocation (_THIS, NSPoint *p) {
     QZ_PrivateCocoaToSDL (this, p);
 }
 
-static void QZ_DoActivate (_THIS)
-{
+void QZ_DoActivate (_THIS) {
+
+    SDL_PrivateAppActive (1, SDL_APPINPUTFOCUS | (QZ_IsMouseInWindow (this) ? SDL_APPMOUSEFOCUS : 0));
+    
     /* Hide the cursor if it was hidden by SDL_ShowCursor() */
     if (!cursor_should_be_visible)
         QZ_HideMouse (this);
@@ -627,9 +633,17 @@ static void QZ_DoActivate (_THIS)
         QZ_PrivateWarpCursor (this, cursor_loc.x, cursor_loc.y);
         QZ_ChangeGrabState (this, QZ_ENABLE_GRAB);
     }
+    else {
+        /* Update SDL's mouse location */
+        NSPoint p;
+        QZ_GetMouseLocation (this, &p);
+        SDL_PrivateMouseMotion (0, 0, p.x, p.y);
+    }
 }
 
-static void QZ_DoDeactivate (_THIS) {
+void QZ_DoDeactivate (_THIS) {
+    
+    SDL_PrivateAppActive (0, SDL_APPINPUTFOCUS | SDL_APPMOUSEFOCUS);
 
     /* Get the current cursor location, for restore on activate */
     QZ_GetMouseLocation (this, &cursor_loc);
@@ -681,26 +695,28 @@ void QZ_RegisterForSleepNotifications (_THIS)
 }
 
 
-// Try to map Quartz mouse buttons to SDL's lingo...
+/* Try to map Quartz mouse buttons to SDL's lingo... */
 static int QZ_OtherMouseButtonToSDL(int button)
 {
     switch (button)
     {
         case 0:
-            return(SDL_BUTTON_LEFT);   // 1
+            return(SDL_BUTTON_LEFT);   /* 1 */
         case 1:
-            return(SDL_BUTTON_RIGHT);  // 3
+            return(SDL_BUTTON_RIGHT);  /* 3 */
         case 2:
-            return(SDL_BUTTON_MIDDLE); // 2
+            return(SDL_BUTTON_MIDDLE); /* 2 */
     }
 
-    // >= 3: skip 4 & 5, since those are the SDL mousewheel buttons.
+    /* >= 3: skip 4 & 5, since those are the SDL mousewheel buttons. */
     return(button + 3);
 }
 
 
 void QZ_PumpEvents (_THIS)
 {
+    static Uint32 screensaverTicks = 0;
+    Uint32 nowTicks;
     int firstMouseEvent;
     CGMouseDelta dx, dy;
 
@@ -709,9 +725,11 @@ void QZ_PumpEvents (_THIS)
     NSRect winRect;
     NSAutoreleasePool *pool;
 
+    if (!SDL_VideoSurface)
+        return;  /* don't do anything if there's no screen surface. */
+
     /* Update activity every five seconds to prevent screensaver. --ryan. */
-    static Uint32 screensaverTicks = 0;
-    Uint32 nowTicks = SDL_GetTicks();
+    nowTicks = SDL_GetTicks();
     if ((nowTicks - screensaverTicks) > 5000)
     {
         UpdateSystemActivity(UsrActivity);
@@ -743,14 +761,9 @@ void QZ_PumpEvents (_THIS)
             BOOL isInGameWin;
             
             #define DO_MOUSE_DOWN(button) do {                                               \
-                            if ( [ NSApp isActive ] ) {                                      \
-                                if ( isInGameWin ) {                                         \
-                                    SDL_PrivateMouseButton (SDL_PRESSED, button, 0, 0);      \
-                                    expect_mouse_up |= 1<<button;                            \
-                                }                                                            \
-                            }                                                                \
-                            else {                                                           \
-                                QZ_DoActivate (this);                                        \
+                            if ( SDL_GetAppState() & SDL_APPMOUSEFOCUS ) {                   \
+                                SDL_PrivateMouseButton (SDL_PRESSED, button, 0, 0);          \
+                                expect_mouse_up |= 1<<button;                                \
                             }                                                                \
                             [ NSApp sendEvent:event ];                                       \
             } while(0)
@@ -770,7 +783,7 @@ void QZ_PumpEvents (_THIS)
 
             switch (type) {
                 case NSLeftMouseDown:
-                    if ( getenv("SDL_HAS3BUTTONMOUSE") ) {
+                    if ( SDL_getenv("SDL_HAS3BUTTONMOUSE") ) {
                         DO_MOUSE_DOWN (SDL_BUTTON_LEFT);
                     } else {
                         if ( NSCommandKeyMask & current_mods ) {
@@ -887,15 +900,34 @@ void QZ_PumpEvents (_THIS)
                     if ( !isInGameWin && (SDL_GetAppState() & SDL_APPMOUSEFOCUS) ) {
                     
                         SDL_PrivateAppActive (0, SDL_APPMOUSEFOCUS);
+                        if (grab_state == QZ_INVISIBLE_GRAB)
+                            /*The cursor has left the window even though it is
+                              disassociated from the mouse (and therefore
+                              shouldn't move): this can happen with Wacom
+                              tablets, and it effectively breaks the grab, since
+                              mouse down events now go to background
+                              applications. The only possibility to avoid this
+                              seems to be talking to the tablet driver
+                              (AppleEvents) to constrain its mapped area to the
+                              window, which may not be worth the effort. For
+                              now, handle the condition more gracefully than
+                              before by reassociating cursor and mouse until the
+                              cursor enters the window again, making it obvious
+                              to the user that the grab is broken.*/
+                            CGAssociateMouseAndMouseCursorPosition (1);
                         if (!cursor_should_be_visible)
                             QZ_ShowMouse (this);
                     }
                     else
-                    if ( isInGameWin && !(SDL_GetAppState() & SDL_APPMOUSEFOCUS) ) {
+                    if ( isInGameWin && (SDL_GetAppState() & (SDL_APPMOUSEFOCUS | SDL_APPINPUTFOCUS)) == SDL_APPINPUTFOCUS ) {
                     
                         SDL_PrivateAppActive (1, SDL_APPMOUSEFOCUS);
                         if (!cursor_should_be_visible)
                             QZ_HideMouse (this);
+                        if (grab_state == QZ_INVISIBLE_GRAB) { /*see comment above*/
+                            QZ_PrivateWarpCursor (this, SDL_VideoSurface->w / 2, SDL_VideoSurface->h / 2);
+                            CGAssociateMouseAndMouseCursorPosition (0);
+                        }
                     }
                     break;
                 case NSScrollWheel:
@@ -921,17 +953,7 @@ void QZ_PumpEvents (_THIS)
                     break;
                 case NSFlagsChanged:
                     break;
-                case NSAppKitDefined:
-                    switch ( [ event subtype ] ) {
-                        case NSApplicationActivatedEventType:
-                            QZ_DoActivate (this);
-                            break;
-                        case NSApplicationDeactivatedEventType:
-                            QZ_DoDeactivate (this);
-                            break;
-                    }
-                    [ NSApp sendEvent:event ];
-                    break;
+                    /* case NSAppKitDefined: break; */
                     /* case NSApplicationDefined: break; */
                     /* case NSPeriodic: break; */
                     /* case NSCursorUpdate: break; */
@@ -947,3 +969,11 @@ void QZ_PumpEvents (_THIS)
     
     [ pool release ];
 }
+
+void QZ_UpdateMouse (_THIS)
+{
+    NSPoint p;
+    QZ_GetMouseLocation (this, &p);
+    SDL_PrivateAppActive (QZ_IsMouseInWindow (this), SDL_APPMOUSEFOCUS);
+    SDL_PrivateMouseMotion (0, 0, p.x, p.y);
+}
