@@ -17,8 +17,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    slouken@devolution.com
+    slouken@libsdl.org
 */
+#include "SDL_config.h"
 
 /*
      File added by Alan Buckley (alan_baa@hotmail.com) for RISC OS compatability
@@ -27,17 +28,11 @@
      Implements RISC OS full screen display.
 */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "SDL.h"
-#include "SDL_error.h"
 #include "SDL_video.h"
 #include "SDL_mouse.h"
-#include "SDL_sysvideo.h"
-#include "SDL_pixels_c.h"
-#include "SDL_events_c.h"
+#include "../SDL_sysvideo.h"
+#include "../SDL_pixels_c.h"
+#include "../../events/SDL_events_c.h"
 
 #include "SDL_riscostask.h"
 #include "SDL_riscosvideo.h"
@@ -198,7 +193,7 @@ SDL_Surface *FULLSCREEN_SetVideoMode(_THIS, SDL_Surface *current,
        if (riscos_backbuffer == 3)
           this->hidden->bank[0] = WIMP_CreateBuffer(width, height, bpp);
        else
-          this->hidden->bank[0] = malloc(height * current->pitch);
+          this->hidden->bank[0] = SDL_malloc(height * current->pitch);
        if (this->hidden->bank[0] == 0)
        {
  	       RISCOS_RestoreWimpMode();
@@ -210,7 +205,7 @@ SDL_Surface *FULLSCREEN_SetVideoMode(_THIS, SDL_Surface *current,
     }
 
     /* Store address of allocated screen bank to be freed later */
-    if (this->hidden->alloc_bank) free(this->hidden->alloc_bank);
+    if (this->hidden->alloc_bank) SDL_free(this->hidden->alloc_bank);
     if (create_back_buffer)
     {
         this->hidden->alloc_bank = this->hidden->bank[0];
@@ -223,8 +218,8 @@ SDL_Surface *FULLSCREEN_SetVideoMode(_THIS, SDL_Surface *current,
 	  this->hidden->alloc_bank = 0;
 
     // Clear both banks to black
-    memset(this->hidden->bank[0], 0, height * current->pitch);
-    memset(this->hidden->bank[1], 0, height * current->pitch);
+    SDL_memset(this->hidden->bank[0], 0, height * current->pitch);
+    SDL_memset(this->hidden->bank[1], 0, height * current->pitch);
 
  	   this->hidden->current_bank = 0;
 	   current->pixels = this->hidden->bank[0];
@@ -280,8 +275,6 @@ void FULLSCREEN_SetDeviceMode(_THIS)
       } else
 	   this->UpdateRects = FULLSCREEN_UpdateRects; /* Default do nothing implementation */
 
-	if (this->SetColors == FULLSCREEN_SetColors) return; /* Already set up */
-
 	this->SetColors   = FULLSCREEN_SetColors;
 
 	this->FlipHWSurface = FULLSCREEN_FlipHWSurface;
@@ -318,7 +311,7 @@ void FULLSCREEN_BuildModeList(_THIS)
 	/* Video memory should be in r[5] */
 	this->info.video_mem = regs.r[5]/1024;
 
-	enumInfo = (unsigned char *)malloc(-regs.r[7]);
+	enumInfo = (unsigned char *)SDL_malloc(-regs.r[7]);
 	if (enumInfo == NULL)
 	{
 		SDL_OutOfMemory();
@@ -354,12 +347,12 @@ void FULLSCREEN_BuildModeList(_THIS)
 		enum_ptr += blockInfo[0];
 	}
 
-	free(enumInfo);
+	SDL_free(enumInfo);
 		
 	/* Sort the mode lists */
 	for ( j=0; j<NUM_MODELISTS; ++j ) {
 		if ( SDL_nummodes[j] > 0 ) {
-			qsort(SDL_modelist[j], SDL_nummodes[j], sizeof *SDL_modelist[j], cmpmodes);
+			SDL_qsort(SDL_modelist[j], SDL_nummodes[j], sizeof *SDL_modelist[j], cmpmodes);
 		}
 	}
 }
@@ -368,14 +361,15 @@ static int FULLSCREEN_FlipHWSurface(_THIS, SDL_Surface *surface)
 {
    _kernel_swi_regs regs;
    regs.r[0] = 19;
-   /* Wait for Vsync */
-   _kernel_swi(OS_Byte, &regs, &regs);
 
    FULLSCREEN_SetDisplayBank(this->hidden->current_bank);
    this->hidden->current_bank ^= 1;
    FULLSCREEN_SetWriteBank(this->hidden->current_bank);
    surface->pixels = this->hidden->bank[this->hidden->current_bank];
 
+   /* Wait for Vsync */
+   _kernel_swi(OS_Byte, &regs, &regs);
+
 	return(0);
 }
 
@@ -398,7 +392,7 @@ static void FULLSCREEN_UpdateRectsMemCpy(_THIS, int numrects, SDL_Rect *rects)
          to  = this->hidden->bank[1] + rects->x * xmult + rects->y * pitch;
          for (row = 0; row < rects->h; row++)
          {
-             memcpy(to, from, rects->w * xmult);
+             SDL_memcpy(to, from, rects->w * xmult);
              from += pitch;
              to += pitch;
          }
@@ -595,7 +589,7 @@ static int FULLSCREEN_AddMode(_THIS, int bpp, int w, int h)
 	}
 
 	/* Set up the new video mode rectangle */
-	mode = (SDL_Rect *)malloc(sizeof *mode);
+	mode = (SDL_Rect *)SDL_malloc(sizeof *mode);
 	if ( mode == NULL ) {
 		SDL_OutOfMemory();
 		return(-1);
@@ -608,11 +602,11 @@ static int FULLSCREEN_AddMode(_THIS, int bpp, int w, int h)
 	/* Allocate the new list of modes, and fill in the new mode */
 	next_mode = SDL_nummodes[index];
 	SDL_modelist[index] = (SDL_Rect **)
-	       realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
+	       SDL_realloc(SDL_modelist[index], (1+next_mode+1)*sizeof(SDL_Rect *));
 	if ( SDL_modelist[index] == NULL ) {
 		SDL_OutOfMemory();
 		SDL_nummodes[index] = 0;
-		free(mode);
+		SDL_free(mode);
 		return(-1);
 	}
 	SDL_modelist[index][next_mode] = mode;
@@ -664,8 +658,7 @@ static void FULLSCREEN_EnableEscape()
 /** Store caption in case this is called before we create a window */
 void FULLSCREEN_SetWMCaption(_THIS, const char *title, const char *icon)
 {
-	strncpy(this->hidden->title, title, 255);
-	this->hidden->title[255] = 0;
+	SDL_strlcpy(this->hidden->title, title, SDL_arraysize(this->hidden->title));
 }
 
 /* Set screen mode
@@ -773,7 +766,7 @@ int FULLSCREEN_ToggleFromWimp(_THIS)
 	   this->screen->pixels = this->hidden->bank[0];
 
        /* Copy back buffer to screen memory */
-       memcpy(this->hidden->bank[1], this->hidden->bank[0], width * height * this->screen->format->BytesPerPixel);
+       SDL_memcpy(this->hidden->bank[1], this->hidden->bank[0], width * height * this->screen->format->BytesPerPixel);
 
        FULLSCREEN_SetDeviceMode(this);
        return 1;
